package controllers

import java.util.HashMap
import models.User
import java.util.Map
import java.util.Collection
import utils.Serializer
import com.google.common.base.Optional
import models.Activity
import models.Location
import static utils.DateTimeFormatters.*

class PacemakerAPI
{
  static long userIndex = 0;
  static long activityIndex = 0;
    
  var Map<Long,   User>      userMap       = new HashMap
  var Map<String, User>      userEmailMap  = new HashMap  
  var Map<Long,   Activity>  activityMap   = new HashMap    
  @Property Collection<User> users         = userMap.values
  @Property Serializer       serializer
  
  new()
  {
    userIndex = 0
    activityIndex = 0
  }
  
  def void load() throws Exception
  {
    serializer.read();
    activityIndex = serializer.pop() as Long
    userIndex     = serializer.pop() as Long
    activityMap   = serializer.pop() as Map<Long, Activity>
    userEmailMap  = serializer.pop() as Map<String, User>
    userMap       = serializer.pop() as Map<Long, User>
    users         = userMap.values
  }
  
  def void store() 
  {
    serializer.push(userMap)
    serializer.push(userEmailMap)
    serializer.push(activityMap)
    serializer.push(userIndex)
    serializer.push(activityIndex)
    serializer.write()
  }
  
  def Long createUser (String firstName, String lastName, String email, String password) 
  { 
    userIndex = userIndex + 1
    var user = new User (userIndex, firstName, lastName, email, password)
    userMap.put(userIndex, user);
    userEmailMap.put(user.email, user)
    userIndex
  }
  
   def getUser (Long id)
   {
     userMap.get(id)
   } 
   
   def getUser (String email)
   {
     userEmailMap.get(email)
   }  
   
   def deleteUser (Long id)
   {
     userEmailMap.remove(userMap.get(id))
     userMap.remove(id)
   }
   
   def deleteUser (String email)
   {
     val user = userEmailMap.remove(getUser(email))
     userMap.remove(user.id)
   }
   
  def Activity createActivity(Long id, String type, String location, double distance, String dateStr, String durationStr)
  {
    var Activity activity = null;
    var user = Optional.fromNullable(userMap.get(id))
    if (user.isPresent())
    {
      activityIndex = activityIndex + 1
      activity = new Activity (activityIndex, type, location, distance, parseDateTime(dateStr), parseDuration(durationStr))
      user.get.activities.put(activity.id, activity);
      activityMap.put(activity.id, activity);
    }
    return activity;
  }
  
  def getActivity (Long id)
  {
    activityMap.get(id)
  }
  
  def void addLocation (Long id, float latitude, float longitude)
  {
    val activity = Optional.fromNullable(activityMap.get(id))
    if (activity.isPresent())
    {
      activity.get.route.add(new Location(latitude, longitude));
    }
  }    
}