package controllers

import parsers.Parser

class PacemakerService
{  
  var PacemakerAPI pacemaker
  var Parser       parser
  
  new(PacemakerAPI pacemaker, Parser parser)
  {
    this.pacemaker = pacemaker
    this.parser = parser
  }
  
 def createUser(String firstname, String lastname, String email, String password) 
  {
    val id = pacemaker.createUser(firstname, lastname, email, password)
    new Ok(parser.renderUser(pacemaker.getUser(id)))
  }
  
  def  getUser(Long id) 
  {
  	val user = pacemaker.getUser(id)
  	if (null != user) new Ok(parser.renderUser(user)) else new NotFound("")
  }
 
  def  getUser(String email) 
  {
  	val user = pacemaker.getUser(email)
  	if (null != user) getUser(user.id) else new NotFound("")
  }
  
  def getUsers()
  {
  	new Ok(parser.renderUsers(pacemaker.users))
  }

  def deleteUser(Long id)
  {
   	val user = pacemaker.getUser(id)
   	pacemaker.deleteUser(user?.id)
  	if (null != user) new Ok("") else new NotFound("")
  }
  
  def createActivity(Long id, String type, String location, double distance, String dateStr, String durationStr)
  {
  	if (null != pacemaker.getUser(id))
  	{
  	  pacemaker.createActivity(id, type, location, distance, dateStr, durationStr)
  	  new Ok("")
  	 }
  	 else new NotFound("")
  }
 
  def getActivities(Long id)
  {
  	val user = pacemaker.getUser(id)
  	if (null != user) new Ok(parser.renderActivities(user.activities.values)) else new NotFound("")
  }
  
  def addLocation (Long id, float latitude, float longitude)
  {
  	val activity = pacemaker.getActivity(id)
  	if (null != activity)
  	{
      pacemaker.addLocation(id, latitude, longitude)
      new Ok("")
  	}
  	else new NotFound("")
  }
  
  def listActivities (Long id, String sortBy)
  {
    val activities = pacemaker.getUser(id).activities.values 

    val report = switch (sortBy)
    {
      case "type"       : activities.sortBy[type]
      case "location"   : activities.sortBy[location]
      case "distance"   : activities.sortBy[distance]
      case "date"       : activities.sortBy[starttime]
      case "duration"   : activities.sortBy[duration]
    }
    return new Ok(parser.renderActivities(report))
  }
}