package controllers

import utils.XMLSerializer
import asg.cliche.Command
import asg.cliche.Param
import asg.cliche.ShellFactory
import parsers.AsciiParser
import utils.JSONSerializer
import parsers.Parser

class PacemakerShell 
{
  var PacemakerAPI       paceApi
  var PacemakerService   pacemaker
  val datastore          = "testdatastore";
  val xmlSerializer      = new XMLSerializer(datastore);
  val jsonSerializer     = new JSONSerializer(datastore);
  
  new() 
  {
    paceApi = new PacemakerAPI;
    pacemaker = new PacemakerService (paceApi, new AsciiParser as Parser)
  }

  @Command(description="List all users details")
  def void listUsers ()
  {
    println(pacemaker.getUsers)
  }
  
  @Command(description="Create a new User")
  def void createUser (@Param(name="first name") String firstname, @Param(name="last name") String lastname, 
                       @Param(name="email")      String email,     @Param(name="password")  String password)
  {
    println (pacemaker.createUser(firstname, lastname, email, password))
  }
  
  @Command(description="List a users details")
  def void listUser (@Param(name="email") String email)
  {
  	println (pacemaker.getUser(email))
  }  
  
  @Command(description="List a users details")
  def void listUser (@Param(name="id")  Long id)
  {
  	println (pacemaker.getUser(id))
  }
  
  @Command(description="List a users activities")
  def void listActivities (@Param(name="user id") Long id)
  {
  	println (pacemaker.getActivities(id))
  }  

  @Command(description="Delete a User")
  def void deleteUser (@Param(name="id") Long id)
  {
    println (pacemaker.deleteUser(id))
  }
  
  @Command(description="Add an activity")
  def void addActivity (@Param(name="user-id")  Long   id,       @Param(name="type") String type, 
                        @Param(name="location") String location, @Param(name="distance") double distance,
                        @Param(name="datetime") String dateStr,  @Param(name="duration") String durationStr
                        )
  {
  	try
  	{
      println (pacemaker.createActivity(id, type, location, distance, dateStr, durationStr))		
  	}
  	catch (IllegalArgumentException e)
  	{
  		println ("Date or Duration format error: " + e.message)
  	}
  }
  
  @Command(description="Add Location to an activity")
  def void addLocation (@Param(name="activity-id")  Long  id,   
                        @Param(name="latitude")     float latitude, @Param(name="longitude") float longitude)
  {
    println (pacemaker.addLocation(id, latitude, longitude))
  }
  
  @Command(description="List Activities")
  def void listActivities (@Param(name="userid") Long id, @Param(name="sortBy: type, location, distance, date, duration") String sortBy)
  {
  	 val options = #{"type", "location", "distance", "date", "duration"}
  	 if (options.contains(sortBy))
  	 {
       println (pacemaker.listActivities(id, sortBy))
     }
     else
       println ("usage : la " + options.toString)
  }
  
  @Command(description="Set file format")
  def void changeFileFormat (@Param(name="file format: xml, json") String fileFormat)
  {
    switch (fileFormat)
    {
      case 'xml'  : paceApi.serializer = xmlSerializer 
      case 'json' : paceApi.serializer = jsonSerializer 
    }
  }
  
  @Command(description="Load activities persistent store")
  def void load ()
  {
    paceApi.load
  }
  
  @Command(description="Store activities persistent store")
  def void store ()
  {
    paceApi.store
  }
  
  def static void main(String[] args) throws Exception
  {
    val main = new PacemakerShell()

    val shell = ShellFactory.createConsoleShell("pm", "Welcome to pacemaker-console - ?help for instructions", main)
    shell.commandLoop
    
    main.paceApi.store
  } 
}