package parsers

import com.fasterxml.jackson.databind.ObjectMapper
import models.User
import models.Activity
import java.util.Collection
import com.fasterxml.jackson.annotation.JsonIgnore

interface UserMixin
{
  @JsonIgnore def String getActivities()
  @JsonIgnore def Long getId()
}

interface ActivityMixin
{
  @JsonIgnore def Long getId()
}

class JsonParser extends Parser
{
  val mapper = new ObjectMapper()
  
  new()
  {
    mapper.addMixInAnnotations(typeof(User),     typeof(UserMixin))
    mapper.addMixInAnnotations(typeof(Activity), typeof(ActivityMixin))    
  }
  
  override renderUser(User user)
  { 
  	 mapper.writerWithDefaultPrettyPrinter.writeValueAsString(user)
  }
  
  override renderActivities(Collection<Activity> activities)
  {
    if (activities.size > 0)
    {
  	 mapper.writerWithDefaultPrettyPrinter.writeValueAsString(activities)
    }
  }
  
  override renderUsers(Collection<User> users)
  {
    mapper.writerWithDefaultPrettyPrinter.writeValueAsString(users)
  } 
}