package controllers

import static org.junit.Assert.*
import org.junit.Test
import org.junit.Before
import org.junit.After
import controllers.PacemakerAPI
import static models.Fixtures.*
import models.User
import models.Location

class PacemakerAPITest 
{
  var PacemakerAPI pacemaker
  var User homer
  
  @Before
  def void setup()
  {
    pacemaker = new PacemakerAPI;
    users.forEach [pacemaker.createUser(firstname, lastname, email, password)]
    homer    = new User(5l, "homer", "simpson", "homer@simpson.com",  "secret")
  }
  
  @After
  def void tearDown()
  {
    pacemaker = null
    homer     = null
  }
  
  @Test
  def void testUser()
  {
    assertEquals (users.size, pacemaker.users.size)
    val id = pacemaker.createUser("homer", "simpson", "homer@simpson.com", "secret")
    assertEquals (users.size+1, pacemaker.users.size)
    assertEquals (homer, pacemaker.getUser(id))
  }  
  
  @Test
  def void testUsers()
  {
    assertEquals (users.size, pacemaker.users.size)

    assertEquals (users.size, pacemaker.users.size)
    users.forEach [ val eachUser = pacemaker.getUser(email)
                    assertEquals (it, eachUser)
                    assertNotSame(it, eachUser) ] 
  }   
  
  @Test def void testDeleteUsers()
  {
    assertEquals (users.length, pacemaker.users.size)
    val user = pacemaker.getUser("marge@simpson.com")
    pacemaker.deleteUser(user.id)
    assertEquals (users.length-1, pacemaker.users.size)  
  }  
  
  @Test def void testAddActivity()
  {
  	val activity = pacemaker.createActivity(pacemaker.getUser("marge@simpson.com").id,  "run", "springfield", 5, "11:2:2012 9:00:00", "20:00:00")
  	
    val returnedActivity = pacemaker.getActivity(activity.id);
    assertEquals(activity,  returnedActivity);
  }   
  
  @Test def void testAddActivityWithSingleLocation()
  {
    val activity = pacemaker.createActivity(pacemaker.getUser("marge@simpson.com").id,  "run", "springfield", 5, "11:2:2012 9:00:00", "20:00:00")
  	
    val location = new Location(23.3f, 33.3f)
   
    val returnedActivity = pacemaker.getActivity(activity.id)
    
    pacemaker.addLocation(returnedActivity.id, location.latitude, location.longitude)
    
    val otherActivity = pacemaker.getActivity(returnedActivity.id)
    assertEquals (1, otherActivity.route.size)
    assertEquals(0.0001, location.latitude,  otherActivity.route.get(0).latitude)
    assertEquals(0.0001, location.longitude, otherActivity.route.get(0).longitude);  
  }   
  
  @Test def void testAddActivityWithMultipleLocation()
  {
  	val locations = newArrayList
  	 (
      new Location(23.3f, 33.3f),
      new Location(34.4f, 45.2f),  
      new Location(25.3f, 34.3f),
      new Location(44.4f, 23.3f)       
     )
     
    val activity = pacemaker.createActivity(pacemaker.getUser("marge@simpson.com").id,  "run", "springfield", 5, "11:2:2012 9:00:00", "20:00:00")
     
    val returnedActivity = pacemaker.getActivity(activity.id)

    locations.forEach[ pacemaker.addLocation(returnedActivity.id, latitude, longitude)]    
    
    val otherActivity = pacemaker.getActivity(returnedActivity.id)
    assertEquals (locations.size, otherActivity.route.size)
    assertEquals (locations, otherActivity.route)
   }   
  
}